from mtai.base import MTAIBase


class Transcribe(MTAIBase):
    """
    Transcribe Class used for various transcription-related operations.
    """

    @classmethod
    def list(cls):
        """
        List all transcriptions.

        Returns:
            JSON Response: A JSON response containing the list of all transcriptions.
        """
        return cls().requests.get("/transcribes/list")

    @classmethod
    def get_transcribe_by_id(cls, transcribe_id):
        """
        Get a transcription by its ID.

        Args:
            transcribe_id (str): The ID of the transcription to retrieve.

        Returns:
            JSON Response: A JSON response containing the transcription data.
        """
        return cls().requests.get(f"/transcribes/retrieve/{transcribe_id}")

    @classmethod
    def create_transcribe_from_audio_url(cls, audio_url, services):
        """
        Create a transcription from an audio URL.

        Args:
            audio_url (str): The URL of the audio file to transcribe.
            services (list of str): A list of services to use for transcription.

        Returns:
            JSON Response: A JSON response containing the result of the transcription.
        """
        return cls().requests.post(
            "/transcribes/transcribe-audio-url",
            data={"audio_url": audio_url, "services": services},
        )

    @classmethod
    def create_transcribe_from_media_file(cls, media_file, services):
        """
        Create a transcription from a media file.

        Args:
            media_file (str): The path or identifier of the media file to transcribe.
            services (list of str): A list of services to use for transcription.

        Returns:
            JSON Response: A JSON response containing the result of the transcription.
        """
        return cls().requests.post(
            "/transcribes/transcribe-media-file",
            data={"media_file": media_file, "services": services},
        )

    @classmethod
    def create_transcribe_from_youtube_video(cls, youtube_url, services):
        """
        Create a transcription from a YouTube video.

        Args:
            youtube_url (str): The URL of the YouTube video to transcribe.
            services (list of str): A list of services to use for transcription.

        Returns:
            JSON Response: A JSON response containing the result of the transcription.
        """
        return cls().requests.post(
            "/transcribes/transcribe-youtube-audio",
            data={"youtube_url": youtube_url, "services": services},
        )

    @classmethod
    def delete_transcribe_by_id(cls, transcribe_id):
        """
        Delete a transcription by its ID.

        Args:
            transcribe_id (str): The ID of the transcription to delete.

        Returns:
            JSON Response: A JSON response indicating the result of the deletion.
        """
        return cls().requests.delete(f"/transcribes/delete/{transcribe_id}")



from mtai.base import MTAIBase


class Bio(MTAIBase):
    """A class to handle operations related to bios."""

    @classmethod
    def list(cls):
        """
        List all bios.

        Returns:
            JSON: A JSON response containing the list of bios.
        """
        return cls().requests.get("/bios/list")

    @classmethod
    def create_mentor_mentee_bio(
        cls, country, job_title, interests, is_mentor=False, max_length=300
    ):
        """
        Create a new bio with the country, job_title, interests, is_mentor, max_length.

        Args:
            country (str): The user's country.
            job_title (str): The user's job title.
            interests (str): The user's interests.
            is_mentor (bool): Whether the user is a mentor.
            max_length (int): The maximum length.

        Returns:
            JSON: A JSON response containing the created bio.
        """
        bio_data = {
            "country": country,
            "job_title": job_title,
            "interests": interests,
            "is_mentor": is_mentor,
            "max_length": max_length,
        }
        return cls().requests.post("/bios/mentor-mentee-bio-create", json=bio_data)

    @classmethod
    def create_bio_from_text(cls, text, output_format, max_length=300):
        """
        Create a new bio from the text, output_format, max_length.

        Args:
            text (str): The text to be used.
            output_format (str): The way the results should be presented.
            max_length (int): The maximum length.

        Returns:
            JSON: A JSON response containing the created bio.
        """
        bio_data = {
            "text": text,
            "output_format": output_format,
            "max_length": max_length,
        }
        return cls().requests.post("/bios/bio-text-create", json=bio_data)

    @classmethod
    def retrieve_bio_by_id(cls, id):
        """
        Retrieve a bio by its ID.

        Args:
            id (str): The ID of the bio to retrieve.

        Returns:
            JSON: A JSON response containing the bio details.
        """
        return cls().requests.get(f"/bios/retrieve/{id}")

    @classmethod
    def delete_bio_by_id(cls, id):
        """
        Delete a bio by its ID.

        Args:
            id (str): The ID of the bio to delete.

        Returns:
            JSON: A JSON response confirming the deletion.
        """
        return cls().requests.delete(f"/bios/delete/{id}")



from mtai.base import MTAIBase


class Prompt(MTAIBase):
    """Prompt Class used across defined."""

    @classmethod
    def list(cls):
        """
        List all prompts.
        Returns:
            JSON Response
        """
        return cls().requests.get("/prompts/list")

    @classmethod
    def ask_question(cls, prompt):
        """
        Create a new response from prompt.
        Args:
            prompt: str
        Returns:
            JSON Response
        """
        return cls().requests.post(
            "/prompts/generate-from-prompt", data={"prompt": prompt}
        )

    @classmethod
    def get_prompt_by_id(cls, promt_id):
        """
        Get a prompt by id.
        Args:
            prompt_id: str
        Returns:
            JSON Response
        """
        return cls().requests.get(f"/prompts/retrieve/{promt_id}")

    @classmethod
    def delete_prompt_by_id(cls, promt_id):
        """
        Delete a prompt by id.
        Args:
            prompt_id: str
        Returns:
            JSON Response
        """
        return cls().requests.delete(f"/prompts/delete/{promt_id}")



from mtai.base import MTAIBase


class Tag(MTAIBase):
    """Tag Class used across defined."""

    @classmethod
    def list(cls):
        """
        List all tags.
        Returns:
            JSON Response
        """
        return cls().requests.get("/tags/list")

    @classmethod
    def create_from_title(cls, title):
        """
        Create a new tag from title.
        Args:
            title: str
        Returns:
            JSON Response
        """
        return cls().requests.post("/tags/create-from-title", data={"title": title})

    @classmethod
    def create_from_title_summary(cls, title, summary, count=5):
        """
        Create a new tag from title and summary.
        Args:
            title: str
            summary: str
            count: int (default 5)
        Returns:
            JSON Response
        """
        return cls().requests.post(
            "/tags/create-tags-from-title-summary",
            data={"title": title, "summary": summary, "count": count},
        )

    @classmethod
    def get_tag_by_id(cls, tag_id):
        """
        Get a tag by id.
        Args:
            tag_id: str
        Returns:
            JSON Response
        """
        return cls().requests.get(f"/tags/retrieve/{tag_id}")

    @classmethod
    def delete_tag_by_id(cls, tag_id):
        """
        Delete a tag by id.
        Args:
            tag_id: str
        Returns:
            JSON Response
        """
        return cls().requests.delete(f"/tags/delete/{tag_id}")



# The `MT` class is a subclass of `MTAIBase` that initializes the `mtai` library with secret key and
# provides access to tags, prompts, and bios functionalities.
"""Entry point defined here."""

from mtai.base import MTAIBase
from mtai.tags import Tag
from mtai.prompts import Prompt
from mtai.bios import Bio
from mtai.transcribes import Transcribe


class MT(MTAIBase):
    """MT Class used across defined."""

    def __init__(self, *args, **kwargs):
        """Initialize mtai with secret key."""
        MTAIBase.__init__(self, *args, **kwargs)

        self.tags = Tag
        self.prompts = Prompt
        self.bios = Bio
        self.transcribes = Transcribe



"""
A setuptools based setup module.
See:
https://www.ktechhub.com/tutorials/how-to-package-a-python-code-and-upload-to-pypi
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

import io
from os import path, getenv
from setuptools import setup, find_packages

# Package meta-data.
NAME = "mtai"
DESCRIPTION = "A python library to consume meditranscribe ai API"
EMAIL = "pypi@ktechhub.com"
AUTHOR = "KtechHub"
REQUIRES_PYTHON = ">=3.8.0"
VERSION = getenv("VERSION", "0.0.1")  # package version
if "v" in VERSION:
    VERSION = VERSION[1:]

# Which packages are required for this module to be executed?
REQUIRED = [
    "requests",
]

here = path.abspath(path.dirname(__file__))


# Get the long description from the README file
with io.open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = "\n" + f.read()

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mediatranscribe/mt-ai-python",
    # Author details
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    license="MIT",
    keywords="mtai mediatranscribe mtaiapi python library",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    packages=find_packages(exclude=["contrib", "docs", "tests"]),
    install_requires=REQUIRED,
    include_package_data=True,
    setup_requires=["wheel"],
    extras_require={
        "test": ["coverage"],
    },
)



"""Base script used across defined."""

import requests
import mtai as api


class Borg:
    """Borg class making class attributes global"""

    _shared_state = {}

    def __init__(self):
        self.__dict__ = self._shared_state


class MTAIBase(Borg):
    """Base Class used across defined."""

    def __init__(self, *args, **kwargs):
        """Initialize mtai with secret key."""
        Borg.__init__(self)
        headers = {}
        secret_key = kwargs.get("secret_key", api.SECRET_KEY)
        if secret_key is not None:
            headers["SECRET-KEY"] = secret_key

        arguments = dict(api_url=api.API_URL, headers=headers)
        if not hasattr(self, "requests"):
            req = MTAIApiRequests(**arguments)
            self._shared_state.update(requests=req)


class MTAIApiRequests(object):
    def __init__(self, api_url=api.API_URL, headers=None):
        """Initialize MTAI API Request object for browsing resource.
        Args:
                api_url: str
                headers: dict
        """
        self.API_BASE_URL = f"{api_url}"
        self.headers = headers

    def _request(self, method, resource_uri, **kwargs):
        """Perform a method on a resource.
        Args:
                method: requests.`method`
                resource_uri: resource endpoint
        Raises:
                HTTPError
        Returns:
                JSON Response
        """
        data = kwargs.get("data")
        params = kwargs.get("params")
        headers = kwargs.get("headers")
        if headers is not None:
            headers.update(self.headers)
        else:
            headers = self.headers
        response = method(
            self.API_BASE_URL + resource_uri, json=data, headers=headers, params=params
        )
        response.raise_for_status()
        return response.json()

    def get(self, endpoint, **kwargs):
        """Get a resource.
        Args:
                endpoint: resource endpoint.
        """
        return self._request(requests.get, endpoint, **kwargs)

    def post(self, endpoint, **kwargs):
        """Create a resource.
        Args:
                endpoint: resource endpoint.
        """
        return self._request(requests.post, endpoint, **kwargs)

    def put(self, endpoint, **kwargs):
        """Update a resource.
        Args:
                endpoint: resource endpoint.
        """
        return self._request(requests.put, endpoint, **kwargs)

    def delete(self, endpoint, **kwargs):
        """Delete a resource.
        Args:
                endpoint: resource endpoint.
        """
        return self._request(requests.delete, endpoint, **kwargs)
